@@ -17,6 +17,7 @@
 # stack.sh
 # ---------
 # install_zaqar
+# install_zaqarui
 # configure_zaqar
 # init_zaqar
 # start_zaqar
@@ -74,10 +75,6 @@ function configure_zaqar {
     [ ! -d $ZAQAR_CONF_DIR ] && sudo mkdir -m 755 -p $ZAQAR_CONF_DIR
     sudo chown $USER $ZAQAR_CONF_DIR
 
-    if [[ -f $ZAQAR_DIR/etc/policy.json.sample ]]; then
-        cp -p $ZAQAR_DIR/etc/policy.json.sample $ZAQAR_POLICY_CONF
-    fi
-
     [ ! -d $ZAQAR_API_LOG_DIR ] && sudo mkdir -m 755 -p $ZAQAR_API_LOG_DIR
     sudo chown $USER $ZAQAR_API_LOG_DIR
 
@@ -101,7 +98,7 @@ function configure_zaqar {
 
     configure_auth_token_middleware $ZAQAR_CONF zaqar $ZAQAR_AUTH_CACHE_DIR
 
-    iniset $ZAQAR_CONF trustee auth_plugin password
+    iniset $ZAQAR_CONF trustee auth_type password
     iniset $ZAQAR_CONF trustee auth_url $KEYSTONE_AUTH_URI
     iniset $ZAQAR_CONF trustee username $ZAQAR_TRUSTEE_USER
     iniset $ZAQAR_CONF trustee password $ZAQAR_TRUSTEE_PASSWORD
@@ -128,10 +125,23 @@ function configure_zaqar {
         iniset $ZAQAR_CONF 'drivers:management_store:sqlalchemy' uri `database_connection_url zaqar`
         iniset $ZAQAR_CONF 'drivers:management_store:sqlalchemy' database zaqar_mgmt
 
+        zaqar-sql-db-manage --config-file $ZAQAR_CONF upgrade head
+
         iniset $ZAQAR_CONF  drivers message_store redis
         iniset $ZAQAR_CONF 'drivers:message_store:redis' uri redis://localhost:6379
         iniset $ZAQAR_CONF 'drivers:message_store:redis' database zaqar
         configure_redis
+    elif [ "$ZAQAR_BACKEND" = 'swift' ] ; then
+        recreate_database zaqar
+        iniset $ZAQAR_CONF  drivers management_store sqlalchemy
+        iniset $ZAQAR_CONF 'drivers:management_store:sqlalchemy' uri `database_connection_url zaqar`
+        iniset $ZAQAR_CONF 'drivers:management_store:sqlalchemy' database zaqar_mgmt
+
+        zaqar-sql-db-manage --config-file $ZAQAR_CONF upgrade head
+
+        iniset $ZAQAR_CONF  drivers message_store swift
+        iniset $ZAQAR_CONF 'drivers:message_store:swift' auth_url $KEYSTONE_AUTH_URI_V3
+        iniset $ZAQAR_CONF 'drivers:message_store:swift' uri swift://zaqar:$SERVICE_PASSWORD@/service
     fi
 
     if is_service_enabled qpid || [ -n "$RABBIT_HOST" ] && [ -n "$RABBIT_PASSWORD" ]; then
@@ -141,14 +151,20 @@ function configure_zaqar {
     iniset_rpc_backend zaqar $ZAQAR_CONF DEFAULT
 
     pip_install uwsgi
+
+    iniset $ZAQAR_UWSGI_CONF uwsgi master true
+    iniset $ZAQAR_UWSGI_CONF uwsgi die-on-term true
+    iniset $ZAQAR_UWSGI_CONF uwsgi exit-on-reload true
     iniset $ZAQAR_UWSGI_CONF uwsgi http $ZAQAR_SERVICE_HOST:$ZAQAR_SERVICE_PORT
-    iniset $ZAQAR_UWSGI_CONF uwsgi harakiri 60
-    iniset $ZAQAR_UWSGI_CONF uwsgi processes 1
+    iniset $ZAQAR_UWSGI_CONF uwsgi processes $API_WORKERS
+    iniset $ZAQAR_UWSGI_CONF uwsgi enable_threads true
     iniset $ZAQAR_UWSGI_CONF uwsgi threads 4
+    iniset $ZAQAR_UWSGI_CONF uwsgi thunder-lock true
+    iniset $ZAQAR_UWSGI_CONF uwsgi buffer-size 65535
     iniset $ZAQAR_UWSGI_CONF uwsgi wsgi-file $ZAQAR_DIR/zaqar/transport/wsgi/app.py
-    iniset $ZAQAR_UWSGI_CONF uwsgi callable app
     iniset $ZAQAR_UWSGI_CONF uwsgi master true
     iniset $ZAQAR_UWSGI_CONF uwsgi add-header "Connection: close"
+    iniset $ZAQAR_UWSGI_CONF uwsgi lazy-apps true
 
     cleanup_zaqar
 }
@@ -194,6 +210,25 @@ function init_zaqar {
 # install_zaqar() - Collect source and prepare
 function install_zaqar {
     setup_develop $ZAQAR_DIR
+
+    if is_service_enabled horizon; then
+        install_zaqarui
+    fi
+}
+
+function install_zaqarui {
+    git_clone $ZAQARUI_REPO $ZAQARUI_DIR $ZAQARUI_BRANCH
+    # NOTE(flwang): Workaround for devstack bug: 1540328
+    # where devstack install 'test-requirements' but should not do it
+    # for zaqar-ui project as it installs Horizon from url.
+    # Remove following two 'mv' commands when mentioned bug is fixed.
+    mv $ZAQARUI_DIR/test-requirements.txt $ZAQARUI_DIR/_test-requirements.txt
+    setup_develop $ZAQARUI_DIR
+    mv $ZAQARUI_DIR/_test-requirements.txt $ZAQARUI_DIR/test-requirements.txt
+    cp -a $ZAQARUI_DIR/zaqar_ui/enabled/* $HORIZON_DIR/openstack_dashboard/local/enabled/
+    if [ -d $ZAQARUI_DIR/zaqar-ui/locale ]; then
+        (cd $ZAQARUI_DIR/zaqar-ui; DJANGO_SETTINGS_MODULE=openstack_dashboard.settings ../manage.py compilemessages)
+    fi
 }
 
 # install_zaqarclient() - Collect source and prepare
@@ -208,11 +243,12 @@ function install_zaqarclient {
 # start_zaqar() - Start running processes, including screen
 function start_zaqar {
     cat $ZAQAR_UWSGI_CONF
-    run_process zaqar-wsgi "uwsgi --ini $ZAQAR_UWSGI_CONF --pidfile2 $ZAQAR_UWSGI_MASTER_PIDFILE"
-    run_process zaqar-websocket "zaqar-server --config-file $ZAQAR_CONF"
+    run_process zaqar-wsgi "$ZAQAR_BIN_DIR/uwsgi --ini $ZAQAR_UWSGI_CONF --pidfile2 $ZAQAR_UWSGI_MASTER_PIDFILE"
+    run_process zaqar-websocket "$ZAQAR_BIN_DIR/zaqar-server --config-file $ZAQAR_CONF"
 
     echo "Waiting for Zaqar to start..."
-    token=$(openstack token issue -c id -f value)
+    local www_authenticate_uri=http://${ZAQAR_SERVICE_HOST}/identity
+    token=$(openstack token issue -c id -f value --os-auth-url ${www_authenticate_uri})
     if ! timeout $SERVICE_TIMEOUT sh -c "while ! wget --no-proxy -q --header=\"Client-ID:$(uuidgen)\" --header=\"X-Auth-Token:$token\" -O- $ZAQAR_SERVICE_PROTOCOL://$ZAQAR_SERVICE_HOST:$ZAQAR_SERVICE_PORT/v2/ping; do sleep 1; done"; then
         die $LINENO "Zaqar did not start"
     fi
@@ -231,7 +267,7 @@ function stop_zaqar {
 function create_zaqar_accounts {
     create_service_user "zaqar"
 
-    if [[ "$KEYSTONE_CATALOG_BACKEND" = 'sql' ]]; then
+    if [[ "$KEYSTONE_IDENTITY_BACKEND" = 'sql' ]]; then
 
         local zaqar_service=$(get_or_create_service "zaqar" \
             "messaging" "Zaqar Service")
@@ -250,6 +286,9 @@ function create_zaqar_accounts {
             "$ZAQAR_SERVICE_PROTOCOL://$ZAQAR_SERVICE_HOST:$ZAQAR_WEBSOCKET_PORT"
     fi
 
+    if [ "$ZAQAR_BACKEND" = 'swift' ] ; then
+        get_or_add_user_project_role ResellerAdmin zaqar service
+    fi
 }
 
 if is_service_enabled zaqar-websocket || is_service_enabled zaqar-wsgi; then
