@@ -222,6 +222,57 @@ class Endpoints(object):
             headers = {'status': 200}
             return response.Response(req, body, headers)
 
+    @api_utils.on_exception_sends_500
+    def queue_purge(self, req):
+        """Purge queue
+
+        :param req: Request instance ready to be sent.
+        :type req: `api.common.Request`
+        :return: resp: Response instance
+        :type: resp: `api.common.Response`
+        """
+        project_id = req._headers.get('X-Project-ID')
+        queue_name = req._body.get('queue_name')
+        resource_types = req._body.get('resource_types', ["messages",
+                                                          "subscriptions"])
+
+        LOG.debug(u'Purge queue - queue: %(queue)s, '
+                  u'project: %(project)s',
+                  {'queue': queue_name, 'project': project_id})
+
+        try:
+            pop_limit = 100
+            if "messages" in resource_types:
+                LOG.debug("Purge all messages under queue %s", queue_name)
+                resp = self._pop_messages(req, queue_name,
+                                          project_id, pop_limit)
+                while resp.get_response()['body']['messages']:
+                    resp = self._pop_messages(req, queue_name,
+                                              project_id, pop_limit)
+
+            if "subscriptions" in resource_types:
+                LOG.debug("Purge all subscriptions under queue %s",
+                          queue_name)
+                resp = self._subscription_controller.list(queue_name,
+                                                          project=project_id)
+                subscriptions = list(next(resp))
+                for sub in subscriptions:
+                    self._subscription_controller.delete(queue_name,
+                                                         sub['id'],
+                                                         project=project_id)
+
+        except storage_errors.QueueDoesNotExist as ex:
+            LOG.exception(ex)
+            headers = {'status': 404}
+            return api_utils.error_response(req, ex, headers)
+        except storage_errors.ExceptionBase as ex:
+            LOG.exception(ex)
+            headers = {'status': 503}
+            return api_utils.error_response(req, ex, headers)
+        else:
+            headers = {'status': 204}
+            return response.Response(req, {}, headers)
+
     # Messages
     @api_utils.on_exception_sends_500
     def message_list(self, req):
@@ -242,6 +293,7 @@ class Endpoints(object):
         try:
             kwargs = api_utils.get_headers(req)
 
+            self._validate.client_id_uuid_safe(req._headers.get('Client-ID'))
             client_uuid = api_utils.get_client_uuid(req)
 
             self._validate.message_listing(**kwargs)
@@ -391,11 +443,8 @@ class Endpoints(object):
 
             queue_max_msg_size = queue_meta.get('_max_messages_post_size',
                                                 None)
-            queue_default_ttl = queue_meta.get('_default_message_ttl', None)
+            queue_default_ttl = queue_meta.get('_default_message_ttl')
 
-            # TODO(flwang): To avoid any unexpected regression issue, we just
-            # leave the _message_post_spec attribute of class as it's. It
-            # should be removed in Newton release.
             if queue_default_ttl:
                 _message_post_spec = (('ttl', int, queue_default_ttl),
                                       ('body', '*', None),)
@@ -420,6 +469,7 @@ class Endpoints(object):
             return api_utils.error_response(req, ex, headers)
 
         try:
+            self._validate.client_id_uuid_safe(req._headers.get('Client-ID'))
             client_uuid = api_utils.get_client_uuid(req)
 
             self._validate.message_posting(messages)
@@ -429,7 +479,8 @@ class Endpoints(object):
                 messages=messages,
                 project=project_id,
                 client_uuid=client_uuid)
-        except (api_errors.BadRequest, validation.ValidationFailed) as ex:
+        except (ValueError, api_errors.BadRequest,
+                validation.ValidationFailed) as ex:
             LOG.debug(ex)
             headers = {'status': 400}
             return api_utils.error_response(req, ex, headers)
@@ -512,7 +563,7 @@ class Endpoints(object):
         project_id = req._headers.get('X-Project-ID')
         queue_name = req._body.get('queue_name')
         message_ids = req._body.get('message_ids')
-        pop_limit = req._body.get('pop_limit')
+        pop_limit = req._body.get('pop')
 
         LOG.debug(u'Messages collection DELETE - queue: %(queue)s,'
                   u'project: %(project)s, messages: %(message_ids)s',
@@ -689,7 +740,7 @@ class Endpoints(object):
         claim_id = req._body.get('claim_id')
 
         LOG.debug(u'Claim update - claim: %(claim_id)s, '
-                  u'queue: %(queue_name)s, project:%(project_id)s' %
+                  u'queue: %(queue_name)s, project:%(project_id)s',
                   {'queue_name': queue_name,
                    'project_id': project_id,
                    'claim_id': claim_id})
@@ -736,7 +787,7 @@ class Endpoints(object):
         claim_id = req._body.get('claim_id')
 
         LOG.debug(u'Claim delete - claim: %(claim_id)s, '
-                  u'queue: %(queue_name)s, project: %(project_id)s' %
+                  u'queue: %(queue_name)s, project: %(project_id)s',
                   {'queue_name': queue_name,
                    'project_id': project_id,
                    'claim_id': claim_id})
