@@ -13,23 +13,27 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import socket
+
 from oslo_log import log
+from osprofiler import opts as profiler_opts
+from osprofiler import profiler
 from stevedore import driver
 
 from zaqar.api import handler
 from zaqar.common import cache as oslo_cache
-from zaqar.common import configs
 from zaqar.common import consts
 from zaqar.common import decorators
 from zaqar.common import errors
+from zaqar.conf import drivers as driver_opts
+from zaqar.conf import opts as opts_tool
 from zaqar.storage import pipeline
 from zaqar.storage import pooling
 from zaqar.storage import utils as storage_utils
 from zaqar.transport import base
+from zaqar.transport.middleware import profile
 from zaqar.transport import validation
 
-from zaqar.i18n import _LE
-
 LOG = log.getLogger(__name__)
 
 
@@ -43,10 +47,15 @@ class Bootstrap(object):
     def __init__(self, conf):
         self.conf = conf
 
-        for group, opts in configs._config_options():
+        for group, opts in opts_tool.list_opts_by_group():
             self.conf.register_opts(opts, group=group)
+        profiler_opts.set_defaults(self.conf)
 
-        self.driver_conf = self.conf[configs._DRIVER_GROUP]
+        # TODO(wangxiyuan): Now the OSprofiler feature in Zaqar only support
+        # wsgi. Websocket part will be added in the future.
+        profile.setup(self.conf, 'Zaqar-server', socket.gethostname())
+
+        self.driver_conf = self.conf[driver_opts.GROUP_NAME]
 
     @decorators.lazy_property(write=False)
     def api(self):
@@ -62,6 +71,9 @@ class Bootstrap(object):
             LOG.debug(u'Storage pooling enabled')
             storage_driver = pooling.DataDriver(self.conf, self.cache,
                                                 self.control)
+            if self.conf.profiler.enabled:
+                storage_driver = profiler.trace_cls("pooling_data_"
+                                                    "driver")(storage_driver)
         else:
             storage_driver = storage_utils.load_storage_driver(
                 self.conf, self.cache, control_driver=self.control)
@@ -109,8 +121,8 @@ class Bootstrap(object):
             return mgr.driver
         except RuntimeError as exc:
             LOG.exception(exc)
-            LOG.error(_LE(u'Failed to load transport driver zaqar.transport.'
-                          u'%(driver)s with args %(args)s'),
+            LOG.error(u'Failed to load transport driver zaqar.transport.'
+                      u'%(driver)s with args %(args)s',
                       {'driver': transport_name, 'args': args})
             raise errors.InvalidDriver(exc)
 
