@@ -16,6 +16,7 @@
 """wsgi transport helpers."""
 
 from distutils import version
+import re
 import uuid
 
 import falcon
@@ -45,7 +46,8 @@ def verify_pre_signed_url(key, req, resp, params):
     if req.method not in methods:
         raise falcon.HTTPNotFound()
 
-    if req.path not in paths:
+    # Support to query single resource with pre-signed url
+    if not any([p for p in paths if re.search(p, req.path)]):
         raise falcon.HTTPNotFound()
 
     try:
@@ -65,17 +67,13 @@ def get_client_uuid(req):
     """Read a required Client-ID from a request.
 
     :param req: A falcon.Request object
-    :raises: HTTPBadRequest if the Client-ID header is missing or
-        does not represent a valid UUID
-    :returns: A UUID object
+    :returns: A UUID object or A string of client id
     """
 
     try:
         return uuid.UUID(req.get_header('Client-ID', required=True))
-
     except ValueError:
-        description = _(u'Malformed hexadecimal UUID.')
-        raise falcon.HTTPBadRequest('Wrong UUID value', description)
+        return req.get_header('Client-ID', required=True)
 
 
 def extract_project_id(req, resp, params):
@@ -110,10 +108,13 @@ def extract_project_id(req, resp, params):
                                     _(u'The header X-PROJECT-ID was missing'))
 
 
-def require_client_id(req, resp, params):
+def require_client_id(validate, req, resp, params):
     """Makes sure the header `Client-ID` is present in the request
 
     Use as a before hook.
+    :param validate: A validator function that will
+        be used to check the format of client id against configured
+        limits.
     :param req: request sent
     :type req: falcon.request.Request
     :param resp: response object to return
@@ -124,9 +125,24 @@ def require_client_id(req, resp, params):
     """
 
     if req.path.startswith('/v1.1/') or req.path.startswith('/v2/'):
-        # NOTE(flaper87): `get_client_uuid` already raises 400
-        # it the header is missing.
-        get_client_uuid(req)
+        try:
+            validate(req.get_header('Client-ID', required=True))
+        except ValueError:
+            description = _(u'Malformed hexadecimal UUID.')
+            raise falcon.HTTPBadRequest('Wrong UUID value', description)
+        except validation.ValidationFailed as ex:
+            raise falcon.HTTPBadRequest(six.text_type(ex))
+    else:
+        # NOTE(wanghao): Since we changed the get_client_uuid to support
+        # other format of client id, so need to check the uuid here for
+        # v1 API.
+        try:
+            client_id = req.get_header('Client-ID')
+            if client_id or client_id == '':
+                uuid.UUID(client_id)
+        except ValueError:
+            description = _(u'Malformed hexadecimal UUID.')
+            raise falcon.HTTPBadRequest('Wrong UUID value', description)
 
 
 def validate_queue_identification(validate, req, resp, params):
@@ -182,7 +198,7 @@ def require_accepts_json(req, resp, params):
     :param params: additional parameters passed to responders
     :type params: dict
     :rtype: None
-    :raises: falcon.HTTPNotAcceptable
+    :raises HTTPNotAcceptable: if the request does not accept JSON
     """
     if not req.client_accepts('application/json'):
         raise falcon.HTTPNotAcceptable(
@@ -212,8 +228,8 @@ def require_content_type_be_non_urlencoded(req, resp, params):
     :param params: additional parameters passed to responders
     :type params: dict
     :rtype: None
-    :raises: falcon.HTTPBadRequest
-
+    :raises HTTPBadRequest: if request has body and "Content-Type" header has
+        "application/x-www-form-urlencoded" value
     """
     if req.content_length is None:
         return
@@ -241,12 +257,13 @@ def inject_context(req, resp, params):
 
     """
     client_id = req.get_header('Client-ID')
-    project_id = params.get('project_id', None)
-    request_id = req.headers.get('X-Openstack-Request-ID'),
+    request_id = req.headers.get('X-Openstack-Request-ID')
     auth_token = req.headers.get('X-AUTH-TOKEN')
-    user = req.headers.get('X-USER-ID')
-    tenant = req.headers.get('X-TENANT-ID')
-
+    project_id = params.get('project_id')
+    user_id = req.headers.get('X-USER-ID')
+    domain_id = req.headers.get('X-DOMAIN-ID')
+    project_domain_id = req.headers.get('X-PROJECT-DOMAIN-ID')
+    user_domain_id = req.headers.get('X-USER-DOMAIN-ID')
     roles = req.headers.get('X-ROLES')
     roles = roles and roles.split(',') or []
 
@@ -254,7 +271,9 @@ def inject_context(req, resp, params):
                                   client_id=client_id,
                                   request_id=request_id,
                                   auth_token=auth_token,
-                                  user=user,
-                                  tenant=tenant,
-                                  roles=roles)
+                                  user_id=user_id,
+                                  roles=roles,
+                                  domain_id=domain_id,
+                                  project_domain_id=project_domain_id,
+                                  user_domain_id=user_domain_id)
     req.env['zaqar.context'] = ctxt
