@@ -12,35 +12,18 @@
 # License for the specific language governing permissions and limitations under
 # the License.
 
-from oslo_config import cfg
 from oslo_log import log as logging
+from osprofiler import profiler
 from stevedore import driver
 from stevedore import extension
 
 from zaqar import common
 from zaqar.common import decorators
-from zaqar.i18n import _
+from zaqar.conf import storage
 from zaqar.storage import base
 
 LOG = logging.getLogger(__name__)
 
-_PIPELINE_RESOURCES = ('queue', 'message', 'claim', 'subscription')
-
-_PIPELINE_CONFIGS = tuple((
-    cfg.ListOpt(resource + '_pipeline', default=[],
-                help=_('Pipeline to use for processing {0} operations. '
-                       'This pipeline will be consumed before calling '
-                       'the storage driver\'s controller methods.')
-                .format(resource))
-    for resource in _PIPELINE_RESOURCES
-))
-
-_PIPELINE_GROUP = 'storage'
-
-
-def _config_options():
-    return [(_PIPELINE_GROUP, _PIPELINE_CONFIGS)]
-
 
 def _get_storage_pipeline(resource_name, conf, *args, **kwargs):
     """Constructs and returns a storage resource pipeline.
@@ -63,10 +46,10 @@ def _get_storage_pipeline(resource_name, conf, *args, **kwargs):
     :returns: A pipeline to use.
     :rtype: `Pipeline`
     """
-    conf.register_opts(_PIPELINE_CONFIGS,
-                       group=_PIPELINE_GROUP)
+    conf.register_opts(storage.ALL_OPTS,
+                       group=storage.GROUP_NAME)
 
-    storage_conf = conf[_PIPELINE_GROUP]
+    storage_conf = conf[storage.GROUP_NAME]
 
     pipeline = []
     for ns in storage_conf[resource_name + '_pipeline']:
@@ -78,14 +61,14 @@ def _get_storage_pipeline(resource_name, conf, *args, **kwargs):
                                        invoke_on_load=True)
             pipeline.append(mgr.driver)
         except RuntimeError as exc:
-            LOG.warning(_(u'Stage %(stage)s could not be imported: %(ex)s'),
+            LOG.warning(u'Stage %(stage)s could not be imported: %(ex)s',
                         {'stage': ns, 'ex': str(exc)})
             continue
 
     return pipeline
 
 
-def _get_builtin_entry_points(resource_name, storage, control_driver):
+def _get_builtin_entry_points(resource_name, storage, control_driver, conf):
     # Load builtin stages
     builtin_entry_points = []
 
@@ -104,7 +87,9 @@ def _get_builtin_entry_points(resource_name, storage, control_driver):
 
     for ext in extensions.extensions:
         builtin_entry_points.append(ext.obj)
-
+    if conf.profiler.enabled and conf.profiler.trace_message_store:
+        return (profiler.trace_cls("stages_controller")
+                (builtin_entry_points))
     return builtin_entry_points
 
 
@@ -138,7 +123,7 @@ class DataDriver(base.DataDriverBase):
     @decorators.lazy_property(write=False)
     def queue_controller(self):
         stages = _get_builtin_entry_points('queue', self._storage,
-                                           self.control_driver)
+                                           self.control_driver, self.conf)
         stages.extend(_get_storage_pipeline('queue', self.conf))
         stages.append(self._storage.queue_controller)
         return common.Pipeline(stages)
@@ -146,9 +131,15 @@ class DataDriver(base.DataDriverBase):
     @decorators.lazy_property(write=False)
     def message_controller(self):
         stages = _get_builtin_entry_points('message', self._storage,
-                                           self.control_driver)
+                                           self.control_driver, self.conf)
         kwargs = {'subscription_controller':
-                  self._storage.subscription_controller}
+                  self._storage.subscription_controller,
+                  'max_notifier_workers':
+                  self.conf.notification.max_notifier_workers,
+                  'require_confirmation':
+                  self.conf.notification.require_confirmation,
+                  'queue_controller':
+                  self._storage.queue_controller}
         stages.extend(_get_storage_pipeline('message', self.conf, **kwargs))
         stages.append(self._storage.message_controller)
         return common.Pipeline(stages)
@@ -156,7 +147,7 @@ class DataDriver(base.DataDriverBase):
     @decorators.lazy_property(write=False)
     def claim_controller(self):
         stages = _get_builtin_entry_points('claim', self._storage,
-                                           self.control_driver)
+                                           self.control_driver, self.conf)
         stages.extend(_get_storage_pipeline('claim', self.conf))
         stages.append(self._storage.claim_controller)
         return common.Pipeline(stages)
@@ -164,7 +155,7 @@ class DataDriver(base.DataDriverBase):
     @decorators.lazy_property(write=False)
     def subscription_controller(self):
         stages = _get_builtin_entry_points('subscription', self._storage,
-                                           self.control_driver)
+                                           self.control_driver, self.conf)
         stages.extend(_get_storage_pipeline('subscription', self.conf))
         stages.append(self._storage.subscription_controller)
         return common.Pipeline(stages)
