@@ -16,12 +16,14 @@
 import heapq
 import itertools
 
-from oslo_config import cfg
 from oslo_log import log
+from osprofiler import profiler
 
 from zaqar.common import decorators
 from zaqar.common import errors as cerrors
 from zaqar.common.storage import select
+from zaqar.conf import pooling_catalog
+from zaqar.i18n import _
 from zaqar import storage
 from zaqar.storage import errors
 from zaqar.storage import pipeline
@@ -29,13 +31,6 @@ from zaqar.storage import utils
 
 LOG = log.getLogger(__name__)
 
-_CATALOG_OPTIONS = (
-    cfg.BoolOpt('enable_virtual_pool', default=False,
-                help=('If enabled, the message_store will be used '
-                      'as the storage for the virtual pool.')),
-)
-
-_CATALOG_GROUP = 'pooling:catalog'
 
 # NOTE(kgriffs): E.g.: 'zaqar-pooling:5083853/my-queue'
 _POOL_CACHE_PREFIX = 'pooling:'
@@ -49,10 +44,6 @@ _POOL_CACHE_PREFIX = 'pooling:'
 _POOL_CACHE_TTL = 10
 
 
-def _config_options():
-    return [(_CATALOG_GROUP, _CATALOG_OPTIONS)]
-
-
 def _pool_cache_key(queue, project=None):
     # NOTE(kgriffs): Use string concatenation for performance,
     # also put project first since it is guaranteed to be
@@ -74,7 +65,11 @@ class DataDriver(storage.DataDriverBase):
 
     def __init__(self, conf, cache, control, control_driver=None):
         super(DataDriver, self).__init__(conf, cache, control_driver)
-        self._pool_catalog = Catalog(conf, cache, control)
+        catalog = Catalog(conf, cache, control)
+        if self.conf.profiler.enabled:
+            catalog = profiler.trace_cls("pooling_catalogue_"
+                                         "controller")(catalog)
+        self._pool_catalog = catalog
 
     @property
     def capabilities(self):
@@ -121,23 +116,40 @@ class DataDriver(storage.DataDriverBase):
 
     @decorators.lazy_property(write=False)
     def queue_controller(self):
-        return QueueController(self._pool_catalog)
+        controller = QueueController(self._pool_catalog)
+        if self.conf.profiler.enabled:
+            return profiler.trace_cls("pooling_queue_controller")(controller)
+        else:
+            return controller
 
     @decorators.lazy_property(write=False)
     def message_controller(self):
-        return MessageController(self._pool_catalog)
+        controller = MessageController(self._pool_catalog)
+        if self.conf.profiler.enabled:
+            return profiler.trace_cls("pooling_message_controller")(controller)
+        else:
+            return controller
 
     @decorators.lazy_property(write=False)
     def claim_controller(self):
-        return ClaimController(self._pool_catalog)
+        controller = ClaimController(self._pool_catalog)
+        if self.conf.profiler.enabled:
+            return profiler.trace_cls("pooling_claim_controller")(controller)
+        else:
+            return controller
 
     @decorators.lazy_property(write=False)
     def subscription_controller(self):
-        return SubscriptionController(self._pool_catalog)
+        controller = SubscriptionController(self._pool_catalog)
+        if self.conf.profiler.enabled:
+            return (profiler.trace_cls("pooling_subscription_controller")
+                    (controller))
+        else:
+            return controller
 
 
 class QueueController(storage.Queue):
-    """Routes operations to a queue controller in the appropriate pool.
+    """Routes operations to get the appropriate queue controller.
 
     :param pool_catalog: a catalog of available pools
     :type pool_catalog: queues.pooling.base.Catalog
@@ -146,20 +158,21 @@ class QueueController(storage.Queue):
     def __init__(self, pool_catalog):
         super(QueueController, self).__init__(None)
         self._pool_catalog = pool_catalog
+        self._mgt_queue_ctrl = self._pool_catalog.control.queue_controller
         self._get_controller = self._pool_catalog.get_queue_controller
 
-    def _list(self, project=None, marker=None,
-              limit=storage.DEFAULT_QUEUES_PER_PAGE, detailed=False):
+    def _list(self, project=None, kfilter={}, marker=None,
+              limit=storage.DEFAULT_QUEUES_PER_PAGE, detailed=False,
+              name=None):
 
         def all_pages():
-            pool = self._pool_catalog.get_default_pool()
-            if pool is None:
-                raise errors.NoPoolFound()
-            yield next(pool.queue_controller.list(
+            yield next(self._mgt_queue_ctrl.list(
                 project=project,
+                kfilter=kfilter,
                 marker=marker,
                 limit=limit,
-                detailed=detailed))
+                detailed=detailed,
+                name=name))
 
         # make a heap compared with 'name'
         ls = heapq.merge(*[
@@ -187,7 +200,7 @@ class QueueController(storage.Queue):
     def _create(self, name, metadata=None, project=None):
         flavor = None
         if isinstance(metadata, dict):
-            flavor = metadata.get('_flavor', None)
+            flavor = metadata.get('_flavor')
 
         self._pool_catalog.register(name, project=project, flavor=flavor)
 
@@ -196,17 +209,15 @@ class QueueController(storage.Queue):
         # however. If between the time we register a queue and go to
         # look it up, the queue is deleted, then this assertion will
         # fail.
-        control = self._get_controller(name, project)
-        if not control:
+        pool = self._pool_catalog.lookup(name, project)
+        if not pool:
             raise RuntimeError('Failed to register queue')
-        return control.create(name, metadata=metadata, project=project)
+        return self._mgt_queue_ctrl.create(name, metadata=metadata,
+                                           project=project)
 
     def _delete(self, name, project=None):
-        # NOTE(cpp-cabrera): If we fail to find a project/queue in the
-        # catalogue for a delete, just ignore it.
-        control = self._get_controller(name, project)
-        if control:
-
+        mqHandler = self._get_controller(name, project)
+        if mqHandler:
             # NOTE(cpp-cabrera): delete from the catalogue first. If
             # zaqar crashes in the middle of these two operations,
             # it is desirable that the entry be missing from the
@@ -217,34 +228,32 @@ class QueueController(storage.Queue):
             # latter case is more difficult to reason about, and may
             # yield 500s in some operations.
             self._pool_catalog.deregister(name, project)
-            ret = control.delete(name, project)
-            return ret
+            mqHandler.delete(name, project)
 
-        return None
+        return self._mgt_queue_ctrl.delete(name, project)
 
     def _exists(self, name, project=None):
-        control = self._get_controller(name, project)
-        if control:
-            return control.exists(name, project=project)
-        return False
+        return self._mgt_queue_ctrl.exists(name, project=project)
 
     def get_metadata(self, name, project=None):
-        control = self._get_controller(name, project)
-        if control:
-            return control.get_metadata(name, project=project)
-        raise errors.QueueDoesNotExist(name, project)
+        return self._mgt_queue_ctrl.get_metadata(name, project=project)
 
     def set_metadata(self, name, metadata, project=None):
-        control = self._get_controller(name, project)
-        if control:
-            return control.set_metadata(name, metadata=metadata,
-                                        project=project)
-        raise errors.QueueDoesNotExist(name, project)
+        # NOTE(gengchc2): If  flavor metadata is modified in queue,
+        # The queue needs to be re-registered to pools, otherwise
+        # the queue flavor parameter is not consistent with the pool.
+        flavor = None
+        if isinstance(metadata, dict):
+            flavor = metadata.get('_flavor')
+        self._pool_catalog.register(name, project=project, flavor=flavor)
+
+        return self._mgt_queue_ctrl.set_metadata(name, metadata=metadata,
+                                                 project=project)
 
     def _stats(self, name, project=None):
-        control = self._get_controller(name, project)
-        if control:
-            return control.stats(name, project=project)
+        mqHandler = self._get_controller(name, project)
+        if mqHandler:
+            return mqHandler.stats(name, project=project)
         raise errors.QueueDoesNotExist(name, project)
 
 
@@ -297,13 +306,15 @@ class MessageController(storage.Message):
 
     def list(self, queue, project=None, marker=None,
              limit=storage.DEFAULT_MESSAGES_PER_PAGE,
-             echo=False, client_uuid=None, include_claimed=False):
+             echo=False, client_uuid=None, include_claimed=False,
+             include_delayed=False):
         control = self._get_controller(queue, project)
         if control:
             return control.list(queue, project=project,
                                 marker=marker, limit=limit,
                                 echo=echo, client_uuid=client_uuid,
-                                include_claimed=include_claimed)
+                                include_claimed=include_claimed,
+                                include_delayed=include_delayed)
         return iter([[]])
 
     def get(self, queue, message_id, project=None):
@@ -409,11 +420,11 @@ class SubscriptionController(storage.Subscription):
             return control.exists(queue, subscription_id,
                                   project=project)
 
-    def confirm(self, queue, subscription_id, project=None, confirm=None):
+    def confirm(self, queue, subscription_id, project=None, confirmed=None):
         control = self._get_controller(queue, project)
         if control:
             return control.confirm(queue, subscription_id,
-                                   project=project, confirm=confirm)
+                                   project=project, confirmed=confirmed)
 
     def get_with_subscriber(self, queue, subscriber, project=None):
         control = self._get_controller(queue, project)
@@ -430,8 +441,9 @@ class Catalog(object):
         self._cache = cache
         self.control = control
 
-        self._conf.register_opts(_CATALOG_OPTIONS, group=_CATALOG_GROUP)
-        self._catalog_conf = self._conf[_CATALOG_GROUP]
+        self._conf.register_opts(pooling_catalog.ALL_OPTS,
+                                 group=pooling_catalog.GROUP_NAME)
+        self._catalog_conf = self._conf[pooling_catalog.GROUP_NAME]
 
         self._pools_ctrl = control.pools_controller
         self._flavor_ctrl = control.flavors_controller
@@ -466,7 +478,7 @@ class Catalog(object):
 
         :returns: pool id
 
-        :raises: `errors.QueueNotMapped`
+        :raises QueueNotMapped: if queue is not mapped
         """
         return self._catalogue_ctrl.get(project, queue)['pool']
 
@@ -490,26 +502,44 @@ class Catalog(object):
         :param flavor: Flavor for the queue (OPTIONAL)
         :type flavor: six.text_type
 
-        :raises: NoPoolFound
+        :raises NoPoolFound: if not found
 
         """
 
-        # NOTE(cpp-cabrera): only register a queue if the entry
-        # doesn't exist
-        if not self._catalogue_ctrl.exists(project, queue):
+        # NOTE(gengchc): if exist, get queue's pool.flavor:
+        # if queue's pool.flavor is different, first delete it and add it.
+        #  Otherwise, if the flavor in the meteredata of the queue is
+        #  modified, the catalog will be inconsistent.
+        if self._catalogue_ctrl.exists(project, queue):
+            catalogue = self._catalogue_ctrl.get(project, queue)
+            oldpoolids = catalogue['pool']
+            oldpool = self._pools_ctrl.get(oldpoolids)
+            oldflavor = oldpool['flavor']
+            msgtmpl = _(u'regiester queue to pool: old flavor: %(oldflavor)s '
+                        ', new flavor: %(flavor)s')
+            LOG.info(msgtmpl,
+                     {'oldflavor': oldflavor, 'flavor': flavor})
+            if oldpool['flavor'] != flavor:
+                self._catalogue_ctrl.delete(project, queue)
 
+        if not self._catalogue_ctrl.exists(project, queue):
             if flavor is not None:
                 flavor = self._flavor_ctrl.get(flavor, project=project)
-                pools = self._pools_ctrl.get_pools_by_group(
-                    group=flavor['pool_group'],
+                pools = self._pools_ctrl.get_pools_by_flavor(
+                    flavor=flavor,
                     detailed=True)
                 pool = select.weighted(pools)
                 pool = pool and pool['name'] or None
+                msgtmpl = _(u'regiester queue to pool: new flavor:%(flavor)s'
+                            ' pool_group:%(pool_group)s')
+                LOG.info(msgtmpl,
+                         {'flavor': flavor.get('name', None),
+                          'pool_group': flavor.get('pool_group', None)})
             else:
                 # NOTE(flaper87): Get pools assigned to the default
                 # group `None`. We should consider adding a `default_group`
                 # option in the future.
-                pools = self._pools_ctrl.get_pools_by_group(detailed=True)
+                pools = self._pools_ctrl.get_pools_by_flavor(detailed=True)
                 pool = select.weighted(pools)
                 pool = pool and pool['name'] or None
 
@@ -523,7 +553,15 @@ class Catalog(object):
                     if self.lookup(queue, project) is not None:
                         return
                     raise errors.NoPoolFound()
+                    msgtmpl = _(u'regiester queue to pool: new flavor: None')
+                    LOG.info(msgtmpl)
 
+            msgtmpl = _(u'regiester queue: project:%(project)s'
+                        ' queue:%(queue)s pool:%(pool)s')
+            LOG.info(msgtmpl,
+                     {'project': project,
+                      'queue': queue,
+                      'pool': pool})
             self._catalogue_ctrl.insert(project, queue, pool)
 
     @_pool_id.purges
