@@ -25,6 +25,7 @@ from zaqar.storage import errors
 from zaqar.storage.redis import models
 from zaqar.storage.redis import scripting
 from zaqar.storage.redis import utils
+from zaqar.storage import utils as s_utils
 
 Message = models.Message
 MessageEnvelope = models.MessageEnvelope
@@ -90,10 +91,16 @@ class MessageController(storage.Message, scripting.Mixin):
         +---------------------+---------+
         |  claim expiry time  |  c.e    |
         +---------------------+---------+
+        |  claim count        |  c.c    |
+        +---------------------+---------+
         |  client uuid        |  u      |
         +---------------------+---------+
         |  created time       |  cr     |
         +---------------------+---------+
+        |  delay expiry time  |  d      |
+        +---------------------+---------+
+        |  body checksum      |  cs      |
+        +---------------------+---------+
 
     4. Messages rank counter (Redis Hash):
 
@@ -110,10 +117,6 @@ class MessageController(storage.Message, scripting.Mixin):
     def _queue_ctrl(self):
         return self.driver.queue_controller
 
-    @decorators.lazy_property(write=False)
-    def _claim_ctrl(self):
-        return self.driver.claim_controller
-
     def _index_messages(self, msgset_key, counter_key, message_ids):
         # NOTE(kgriffs): A watch on a pipe could also be used to ensure
         # messages are inserted in order, but that would be less efficient.
@@ -225,7 +228,7 @@ class MessageController(storage.Message, scripting.Mixin):
     def _list(self, queue, project=None, marker=None,
               limit=storage.DEFAULT_MESSAGES_PER_PAGE,
               echo=False, client_uuid=None,
-              include_claimed=False,
+              include_claimed=False, include_delayed=False,
               to_basic=True):
 
         if not self._queue_ctrl.exists(queue, project):
@@ -264,6 +267,10 @@ class MessageController(storage.Message, scripting.Mixin):
             filters.append(functools.partial(utils.msg_claimed_filter,
                                              now=now))
 
+        if not include_delayed:
+            filters.append(functools.partial(utils.msg_delayed_filter,
+                                             now=now))
+
         if not echo:
             filters.append(functools.partial(utils.msg_echo_filter,
                                              client_uuid=client_uuid))
@@ -283,6 +290,7 @@ class MessageController(storage.Message, scripting.Mixin):
 
         :returns: Number of messages removed
         """
+        claim_ctrl = self.driver.claim_controller
         client = self._client
 
         num_removed = 0
@@ -306,7 +314,7 @@ class MessageController(storage.Message, scripting.Mixin):
                 # here, because we already know the queue and project
                 # scope.
                 queue, project = utils.descope_message_ids_set(msgset_key)
-                self._claim_ctrl._gc(queue, project)
+                claim_ctrl._gc(queue, project)
 
                 offset_mids = 0
 
@@ -345,10 +353,11 @@ class MessageController(storage.Message, scripting.Mixin):
     def list(self, queue, project=None, marker=None,
              limit=storage.DEFAULT_MESSAGES_PER_PAGE,
              echo=False, client_uuid=None,
-             include_claimed=False):
+             include_claimed=False, include_delayed=False):
 
         return self._list(queue, project, marker, limit, echo,
-                          client_uuid, include_claimed)
+                          client_uuid, include_claimed,
+                          include_delayed)
 
     @utils.raises_conn_error
     @utils.retries_on_connection_error
@@ -410,7 +419,6 @@ class MessageController(storage.Message, scripting.Mixin):
 
         message_ids = []
         now = timeutils.utcnow_ts()
-
         with self._client.pipeline() as pipe:
             for msg in messages:
                 prepared_msg = Message(
@@ -419,7 +427,11 @@ class MessageController(storage.Message, scripting.Mixin):
                     client_uuid=client_uuid,
                     claim_id=None,
                     claim_expires=now,
+                    claim_count=0,
+                    delay_expires=now + msg.get('delay', 0),
                     body=msg.get('body', {}),
+                    checksum=s_utils.get_checksum(msg.get('body', None)) if
+                    self.driver.conf.enable_checksum else None
                 )
 
                 prepared_msg.to_redis(pipe)
@@ -439,6 +451,7 @@ class MessageController(storage.Message, scripting.Mixin):
     @utils.raises_conn_error
     @utils.retries_on_connection_error
     def delete(self, queue, message_id, project=None, claim=None):
+        claim_ctrl = self.driver.claim_controller
         if not self._queue_ctrl.exists(queue, project):
             return
 
@@ -468,7 +481,7 @@ class MessageController(storage.Message, scripting.Mixin):
             raise errors.MessageNotClaimed(message_id)
 
         elif msg_claim['id'] != claim:
-            if not self._claim_ctrl._exists(queue, claim, project):
+            if not claim_ctrl._exists(queue, claim, project):
                 raise errors.ClaimDoesNotExist(claim, queue, project)
 
             raise errors.MessageNotClaimedBy(message_id, claim)
@@ -480,15 +493,15 @@ class MessageController(storage.Message, scripting.Mixin):
             pipe.zrem(msgset_key, message_id)
 
             if is_claimed:
-                self._claim_ctrl._del_message(queue, project,
-                                              msg_claim['id'], message_id,
-                                              pipe)
+                claim_ctrl._del_message(queue, project, msg_claim['id'],
+                                        message_id, pipe)
 
             pipe.execute()
 
     @utils.raises_conn_error
     @utils.retries_on_connection_error
     def bulk_delete(self, queue, message_ids, project=None):
+        claim_ctrl = self.driver.claim_controller
         if not self._queue_ctrl.exists(queue, project):
             return
 
@@ -504,9 +517,8 @@ class MessageController(storage.Message, scripting.Mixin):
 
                 msg_claim = self._get_claim(mid)
                 if msg_claim is not None:
-                    self._claim_ctrl._del_message(queue, project,
-                                                  msg_claim['id'], mid,
-                                                  pipe)
+                    claim_ctrl._del_message(queue, project, msg_claim['id'],
+                                            mid, pipe)
             pipe.execute()
 
     @utils.raises_conn_error
@@ -516,8 +528,8 @@ class MessageController(storage.Message, scripting.Mixin):
         # 1. Create a claim.
         # 2. Delete the messages claimed.
         # 3. Delete the claim.
-
-        claim_id, messages = self._claim_ctrl.create(
+        claim_ctrl = self.driver.claim_controller
+        claim_id, messages = claim_ctrl.create(
             queue, dict(ttl=1, grace=0), project, limit=limit)
 
         message_ids = [message['id'] for message in messages]
@@ -525,7 +537,7 @@ class MessageController(storage.Message, scripting.Mixin):
         # NOTE(prashanthr_): Creating a claim controller reference
         # causes a recursive reference. Hence, using the reference
         # from the driver.
-        self._claim_ctrl.delete(queue, claim_id, project)
+        claim_ctrl.delete(queue, claim_id, project)
         return messages
 
 
@@ -533,7 +545,7 @@ def _filter_messages(messages, filters, to_basic, marker):
     """Create a filtering iterator over a list of messages.
 
     The function accepts a list of filters to be filtered
-    before the the message can be included as a part of the reply.
+    before the message can be included as a part of the reply.
     """
     now = timeutils.utcnow_ts()
 
