@@ -13,10 +13,10 @@
 # the License.
 
 import functools
-import uuid
 
 import msgpack
 from oslo_utils import timeutils
+from oslo_utils import uuidutils
 import redis
 
 from zaqar.common import utils as common_utils
@@ -33,9 +33,9 @@ SUBSCRIPTION_IDS_SUFFIX = 'subscriptions'
 
 
 class SubscriptionController(base.Subscription):
-    """Implements subscription resource operations using MongoDB.
+    """Implements subscription resource operations using Redis.
 
-    Subscriptions are unique by project + queue/topic + subscriber.
+    Subscriptions are unique by project + queue + subscriber.
 
     Schema:
       's': source :: six.text_type
@@ -71,6 +71,9 @@ class SubscriptionController(base.Subscription):
             ttl = int(record[2])
             expires = int(record[3])
             created = expires - ttl
+            is_confirmed = True
+            if len(record) == 6:
+                is_confirmed = record[5] == str(True)
             ret = {
                 'id': sid,
                 'source': record[0],
@@ -78,6 +81,7 @@ class SubscriptionController(base.Subscription):
                 'ttl': ttl,
                 'age': now - created,
                 'options': self._unpacker(record[4]),
+                'confirmed': is_confirmed,
             }
             marker_next['next'] = sid
 
@@ -89,8 +93,10 @@ class SubscriptionController(base.Subscription):
     @utils.raises_conn_error
     @utils.retries_on_connection_error
     def get(self, queue, subscription_id, project=None):
-        subscription = SubscriptionEnvelope.from_redis(subscription_id,
-                                                       self._client)
+        subscription = None
+        if self.exists(queue, subscription_id, project):
+            subscription = SubscriptionEnvelope.from_redis(subscription_id,
+                                                           self._client)
         if subscription:
             now = timeutils.utcnow_ts()
             return subscription.to_basic(now)
@@ -100,7 +106,7 @@ class SubscriptionController(base.Subscription):
     @utils.raises_conn_error
     @utils.retries_on_connection_error
     def create(self, queue, subscriber, ttl, options, project=None):
-        subscription_id = str(uuid.uuid4())
+        subscription_id = uuidutils.generate_uuid()
         subset_key = utils.scope_subscription_ids_set(queue,
                                                       project,
                                                       SUBSCRIPTION_IDS_SUFFIX)
@@ -108,6 +114,7 @@ class SubscriptionController(base.Subscription):
         source = queue
         now = timeutils.utcnow_ts()
         expires = now + ttl
+        confirmed = False
 
         subscription = {'id': subscription_id,
                         's': source,
@@ -115,7 +122,8 @@ class SubscriptionController(base.Subscription):
                         't': ttl,
                         'e': expires,
                         'o': self._packer(options),
-                        'p': project}
+                        'p': project,
+                        'c': confirmed}
 
         try:
             # Pipeline ensures atomic inserts.
@@ -150,8 +158,9 @@ class SubscriptionController(base.Subscription):
         try:
             sub_ids = (q for q in self._client.zrange(subset_key, 0, -1))
             for s_id in sub_ids:
-                subscription = self._client.hmget(s_id, ['s', 'u', 't', 'o'])
-                if subscription == [None, None, None, None]:
+                subscription = self._client.hmget(s_id,
+                                                  ['s', 'u', 't', 'o', 'c'])
+                if subscription == [None, None, None, None, None]:
                     # NOTE(flwang): Under this check, that means the
                     # subscription has been expired. So redis can't get
                     # the subscription but the id is still there. So let's
@@ -187,7 +196,7 @@ class SubscriptionController(base.Subscription):
         subscription_to_update = self.get(queue, subscription_id,
                                           project=project)
 
-        new_subscriber = fields.get('u', None)
+        new_subscriber = fields.get('u')
 
         # Let's do some checks to prevent subscription duplication.
         if new_subscriber:
@@ -201,11 +210,11 @@ class SubscriptionController(base.Subscription):
 
         # NOTE(Eva-i): if there are new options, we need to pack them before
         # sending to the database.
-        new_options = fields.get('o', None)
+        new_options = fields.get('o')
         if new_options is not None:
             fields['o'] = self._packer(new_options)
 
-        new_ttl = fields.get('t', None)
+        new_ttl = fields.get('t')
         if new_ttl is not None:
             now = timeutils.utcnow_ts()
             expires = now + new_ttl
@@ -223,8 +232,38 @@ class SubscriptionController(base.Subscription):
     def delete(self, queue, subscription_id, project=None):
         subset_key = utils.scope_subscription_ids_set(queue, project,
                                                       SUBSCRIPTION_IDS_SUFFIX)
-        # NOTE(prashanthr_): Pipelining is used to mitigate race conditions
+
+        if self._client.zrank(subset_key, subscription_id) is not None:
+            # NOTE(prashanthr_): Pipelining is used to mitigate race conditions
+            with self._client.pipeline() as pipe:
+                pipe.zrem(subset_key, subscription_id)
+                pipe.delete(subscription_id)
+                pipe.execute()
+
+    @utils.raises_conn_error
+    @utils.retries_on_connection_error
+    def get_with_subscriber(self, queue, subscriber, project=None):
+        subset_key = utils.scope_subscription_ids_set(queue,
+                                                      project,
+                                                      SUBSCRIPTION_IDS_SUFFIX)
+        sub_ids = (q for q in self._client.zrange(subset_key, 0, -1))
+        for s_id in sub_ids:
+            subscription = self._client.hmget(s_id,
+                                              ['s', 'u', 't', 'o', 'c'])
+            if subscription[1] == subscriber:
+                subscription = SubscriptionEnvelope.from_redis(s_id,
+                                                               self._client)
+                now = timeutils.utcnow_ts()
+                return subscription.to_basic(now)
+
+    @utils.raises_conn_error
+    @utils.retries_on_connection_error
+    def confirm(self, queue, subscription_id, project=None, confirmed=True):
+        # Let's get our subscription by ID. If it does not exist,
+        # SubscriptionDoesNotExist error will be raised internally.
+        self.get(queue, subscription_id, project=project)
+
+        fields = {'c': confirmed}
         with self._client.pipeline() as pipe:
-            pipe.zrem(subset_key, subscription_id)
-            pipe.delete(subscription_id)
+            pipe.hmset(subscription_id, fields)
             pipe.execute()
