@@ -26,12 +26,13 @@ import six
 from testtools import matchers
 
 from zaqar.common import cache as oslo_cache
-from zaqar.common import configs
+from zaqar.conf import default
+from zaqar.conf import drivers_management_store_mongodb
+from zaqar.conf import drivers_message_store_mongodb
 from zaqar import storage
 from zaqar.storage import errors
 from zaqar.storage import mongodb
 from zaqar.storage.mongodb import controllers
-from zaqar.storage.mongodb import options
 from zaqar.storage.mongodb import utils
 from zaqar.storage import pooling
 from zaqar import tests as testing
@@ -51,12 +52,12 @@ class MongodbSetupMixin(object):
             self.driver.connection.drop_database(db)
 
     def _prepare_conf(self):
-        if options.MESSAGE_MONGODB_GROUP in self.conf:
-            self.config(options.MESSAGE_MONGODB_GROUP,
+        if drivers_message_store_mongodb.GROUP_NAME in self.conf:
+            self.config(drivers_message_store_mongodb.GROUP_NAME,
                         database=uuid.uuid4().hex)
 
-        if options.MANAGEMENT_MONGODB_GROUP in self.conf:
-            self.config(options.MANAGEMENT_MONGODB_GROUP,
+        if drivers_management_store_mongodb.GROUP_NAME in self.conf:
+            self.config(drivers_management_store_mongodb.GROUP_NAME,
                         database=uuid.uuid4().hex)
 
 
@@ -67,10 +68,10 @@ class MongodbUtilsTest(MongodbSetupMixin, testing.TestBase):
     def setUp(self):
         super(MongodbUtilsTest, self).setUp()
 
-        self.conf.register_opts(options.MESSAGE_MONGODB_OPTIONS,
-                                group=options.MESSAGE_MONGODB_GROUP)
+        self.conf.register_opts(drivers_message_store_mongodb.ALL_OPTS,
+                                group=drivers_message_store_mongodb.GROUP_NAME)
 
-        self.mongodb_conf = self.conf[options.MESSAGE_MONGODB_GROUP]
+        self.mongodb_conf = self.conf[drivers_message_store_mongodb.GROUP_NAME]
 
         MockDriver = collections.namedtuple('MockDriver', 'mongodb_conf')
 
@@ -110,8 +111,6 @@ class MongodbUtilsTest(MongodbSetupMixin, testing.TestBase):
         self.assertRaises(ValueError, utils.calculate_backoff, 0, 10, -2, 0)
         self.assertRaises(ValueError, utils.calculate_backoff, 0, 10, 2, -1)
 
-        self.assertRaises(ValueError, utils.calculate_backoff, -2, -10, 2, 0)
-        self.assertRaises(ValueError, utils.calculate_backoff, 2, -10, 2, 0)
         self.assertRaises(ValueError, utils.calculate_backoff, -2, 10, 2, 0)
         self.assertRaises(ValueError, utils.calculate_backoff, -1, 10, 2, 0)
         self.assertRaises(ValueError, utils.calculate_backoff, 10, 10, 2, 0)
@@ -154,7 +153,7 @@ class MongodbDriverTest(MongodbSetupMixin, testing.TestBase):
     def setUp(self):
         super(MongodbDriverTest, self).setUp()
 
-        self.conf.register_opts(configs._GENERAL_OPTIONS)
+        self.conf.register_opts(default.ALL_OPTS)
         self.config(unreliable=False)
         oslo_cache.register_config(self.conf)
 
@@ -495,34 +494,6 @@ class MongodbSubscriptionTests(MongodbSetupMixin,
     controller_class = controllers.SubscriptionController
     control_driver_class = mongodb.ControlDriver
 
-    def test_confirm(self):
-        s_id = self.subscription_controller.create(self.source,
-                                                   self.subscriber,
-                                                   self.ttl,
-                                                   self.options,
-                                                   project=self.project)
-        self.addCleanup(self.subscription_controller.delete, self.source,
-                        s_id, self.project)
-        subscription = self.subscription_controller.get(self.source, s_id,
-                                                        project=self.project)
-
-        self.assertEqual(False, subscription['confirmed'])
-
-        self.subscription_controller.confirm(self.source, s_id,
-                                             project=self.project,
-                                             confirm=True)
-        subscription = self.subscription_controller.get(self.source, s_id,
-                                                        project=self.project)
-
-        self.assertEqual(True, subscription['confirmed'])
-
-    def test_confirm_with_nonexist_subscription(self):
-        s_id = 'fake-id'
-        self.assertRaises(errors.SubscriptionDoesNotExist,
-                          self.subscription_controller.confirm,
-                          self.source, s_id, project=self.project, confirm=True
-                          )
-
 
 #
 # TODO(kgriffs): Do these need database purges as well as those above?
@@ -537,10 +508,14 @@ class MongodbPoolsTests(base.PoolsControllerTest):
 
     def setUp(self):
         super(MongodbPoolsTests, self).setUp()
+        self.pools_controller.create(self.pool, 100, 'localhost',
+                                     group=self.pool_group, options={})
 
     def tearDown(self):
         super(MongodbPoolsTests, self).tearDown()
 
+    # NOTE(gengchc2): remove test_delete_pool_used_by_flavor in Rocky release
+    # and use test_delete_pool_used_by_flavor1 instead for pool_group removal.
     def test_delete_pool_used_by_flavor(self):
         self.flavors_controller.create('durable', self.pool_group,
                                        project=self.project,
@@ -549,6 +524,19 @@ class MongodbPoolsTests(base.PoolsControllerTest):
         with testing.expect(errors.PoolInUseByFlavor):
             self.pools_controller.delete(self.pool)
 
+    # NOTE(gengchc2): Unittest for new flavor configure scenario.
+    def test_delete_pool_used_by_flavor1(self):
+        self.flavors_controller.create(self.flavor,
+                                       project=self.project,
+                                       capabilities={})
+        self.pools_controller.update(self.pool1, flavor=self.flavor)
+
+        with testing.expect(errors.PoolInUseByFlavor):
+            self.pools_controller.delete(self.pool1)
+
+    # NOTE(gengchc2): remove test_mismatching_capabilities_fifo in Rocky
+    # release and use test_mismatching_capabilities_fifo1 instead for
+    # pool_group removal.
     def test_mismatching_capabilities_fifo(self):
         with testing.expect(errors.PoolCapabilitiesMismatch):
             self.pools_controller.create(str(uuid.uuid1()),
@@ -556,6 +544,36 @@ class MongodbPoolsTests(base.PoolsControllerTest):
                                          group=self.pool_group,
                                          options={})
 
+    # NOTE(gengchc2): Unittest for new flavor configure scenario.
+    def test_mismatching_capabilities_fifo1(self):
+        with testing.expect(errors.PoolCapabilitiesMismatch):
+            self.pools_controller.create(str(uuid.uuid1()),
+                                         100, 'mongodb.fifo://localhost',
+                                         flavor=self.flavor,
+                                         options={})
+
+    # NOTE(gengchc2): remove test_mismatching_capabilities in Rocky release
+    #  and use test_mismatching_capabilities1 instead for pool_group removal.
+    def test_mismatching_capabilities(self):
+        # NOTE(gengchc2): This test is used for testing mismatchming
+        # capabilities in pool with group
+        with testing.expect(errors.PoolCapabilitiesMismatch):
+            self.pools_controller.create(str(uuid.uuid1()),
+                                         100, 'redis://localhost',
+                                         group=self.pool_group,
+                                         options={})
+
+    def test_mismatching_capabilities1(self):
+        # NOTE(gengchc2): This test is used for testing mismatchming
+        # capabilities in pool with flavor
+        with testing.expect(errors.PoolCapabilitiesMismatch):
+            self.pools_controller.create(str(uuid.uuid1()),
+                                         100, 'redis://localhost',
+                                         flavor=self.flavor,
+                                         options={})
+
+    # NOTE(gengchc2): remove test_duplicate_uri in Rocky release and
+    #  use test_duplicate_uri1 instead for pool_group removal.
     def test_duplicate_uri(self):
         with testing.expect(errors.PoolAlreadyExists):
             # The url 'localhost' is used in setUp(). So reusing the uri
@@ -563,6 +581,14 @@ class MongodbPoolsTests(base.PoolsControllerTest):
             self.pools_controller.create(str(uuid.uuid1()), 100, 'localhost',
                                          group=str(uuid.uuid1()), options={})
 
+    # NOTE(gengchc2): Unittest for new flavor configure scenario.
+    def test_duplicate_uri1(self):
+        with testing.expect(errors.PoolAlreadyExists):
+            # The url 'localhost' is used in setUp(). So reusing the uri
+            # 'localhost' here will raise PoolAlreadyExists.
+            self.pools_controller.create(str(uuid.uuid1()), 100, 'localhost',
+                                         flavor=str(uuid.uuid1()), options={})
+
 
 @testing.requires_mongodb
 class MongodbCatalogueTests(base.CatalogueControllerTest):
@@ -607,6 +633,8 @@ class PooledClaimsTests(base.ClaimControllerTest):
         self.skip("Fix sqlalchemy driver")
 
 
+# NOTE(gengchc2): remove MongodbFlavorsTest in Rocky release and
+# use MongodbFlavorsTest1 instead for pool_group removal.
 @testing.requires_mongodb
 class MongodbFlavorsTest(base.FlavorsControllerTest):
     driver_class = mongodb.ControlDriver
@@ -617,3 +645,16 @@ class MongodbFlavorsTest(base.FlavorsControllerTest):
     def setUp(self):
         super(MongodbFlavorsTest, self).setUp()
         self.addCleanup(self.controller.drop_all)
+
+
+# NOTE(gengchc2): Unittest for new flavor configure scenario.
+@testing.requires_mongodb
+class MongodbFlavorsTest1(base.FlavorsControllerTest1):
+    driver_class = mongodb.ControlDriver
+    controller_class = controllers.FlavorsController
+    control_driver_class = mongodb.ControlDriver
+    config_file = 'wsgi_mongodb.conf'
+
+    def setUp(self):
+        super(MongodbFlavorsTest1, self).setUp()
+        self.addCleanup(self.controller.drop_all)
