@@ -24,7 +24,7 @@ from zaqar.tests.unit.transport.wsgi import base
 
 
 @contextlib.contextmanager
-def flavor(test, name, pool_group):
+def flavor(test, name, pool_group, capabilities={}):
     """A context manager for constructing a flavor for use in testing.
 
     Deletes the flavor after exiting the context.
@@ -32,19 +32,20 @@ def flavor(test, name, pool_group):
     :param test: Must expose simulate_* methods
     :param name: Name for this flavor
     :type name: six.text_type
-    :type pool: six.text_type
+    :type pool_group: six.text_type
+    :type capabilities: dict
     :returns: (name, uri, capabilities)
     :rtype: see above
 
     """
 
-    doc = {'pool_group': pool_group}
+    doc = {'pool_group': pool_group, 'capabilities': capabilities}
     path = test.url_prefix + '/flavors/' + name
 
     test.simulate_put(path, body=jsonutils.dumps(doc))
 
     try:
-        yield name, pool_group
+        yield name, pool_group, capabilities
 
     finally:
         test.simulate_delete(path)
@@ -65,21 +66,21 @@ def flavors(test, count, pool_group):
     """
 
     base = test.url_prefix + '/flavors/'
-    args = sorted([(base + str(i), str(i)) for i in range(count)],
-                  key=lambda tup: tup[1])
-    for path, _ in args:
-        doc = {'pool_group': pool_group}
+    args = sorted([(base + str(i), {str(i): i}, str(i)) for i in range(count)],
+                  key=lambda tup: tup[2])
+    for path, capabilities, _ in args:
+        doc = {'pool_group': pool_group, 'capabilities': capabilities}
         test.simulate_put(path, body=jsonutils.dumps(doc))
 
     try:
         yield args
     finally:
-        for path, _ in args:
+        for path, _, _ in args:
             test.simulate_delete(path)
 
 
 @ddt.ddt
-class TestFlavorsMongoDB(base.V2Base):
+class TestFlavorsMongoDB(base.V1_1Base):
 
     config_file = 'wsgi_mongodb_pooled.conf'
 
@@ -94,7 +95,7 @@ class TestFlavorsMongoDB(base.V2Base):
         self.pool_path = self.url_prefix + '/pools/' + self.pool
         self.pool_doc = {'weight': 100,
                          'group': self.pool_group,
-                         'uri': self.mongodb_url + '/test'}
+                         'uri': self.mongodb_url}
         self.simulate_put(self.pool_path, body=jsonutils.dumps(self.pool_doc))
 
         self.flavor = 'test-flavor'
@@ -107,8 +108,8 @@ class TestFlavorsMongoDB(base.V2Base):
         self.simulate_delete(self.queue_path)
         self.simulate_delete(self.flavor_path)
         self.assertEqual(falcon.HTTP_204, self.srmock.status)
-        self.simulate_delete(self.pool_path)
 
+        self.simulate_delete(self.pool_path)
         super(TestFlavorsMongoDB, self).tearDown()
 
     def test_put_flavor_works(self):
@@ -126,10 +127,10 @@ class TestFlavorsMongoDB(base.V2Base):
         self.assertEqual(falcon.HTTP_400, self.srmock.status)
 
     @ddt.data(1, 2**32+1, [])
-    def test_put_raises_if_invalid_pool(self, pool_group):
+    def test_put_raises_if_invalid_pool(self, pool):
         path = self.url_prefix + '/flavors/' + str(uuid.uuid1())
         self.simulate_put(path,
-                          body=jsonutils.dumps({'pool_group': pool_group}))
+                          body=jsonutils.dumps({'pool_group': pool}))
         self.assertEqual(falcon.HTTP_400, self.srmock.status)
 
     @ddt.data(-1, 'wee', [])
@@ -172,18 +173,18 @@ class TestFlavorsMongoDB(base.V2Base):
         self.assertEqual(falcon.HTTP_204, self.srmock.status)
 
         self.simulate_get(self.flavor_path)
-        self.assertEqual(falcon.HTTP_404, self.srmock.status)
+        self.assertEqual(self.srmock.status, falcon.HTTP_404)
 
     def test_get_nonexisting_raises_404(self):
         self.simulate_get(self.url_prefix + '/flavors/nonexisting')
-        self.assertEqual(falcon.HTTP_404, self.srmock.status)
+        self.assertEqual(self.srmock.status, falcon.HTTP_404)
 
-    def _flavor_expect(self, flavor, xhref, xpool_group):
+    def _flavor_expect(self, flavor, xhref, xpool):
         self.assertIn('href', flavor)
         self.assertIn('name', flavor)
         self.assertEqual(xhref, flavor['href'])
         self.assertIn('pool_group', flavor)
-        self.assertEqual(xpool_group, flavor['pool_group'])
+        self.assertEqual(xpool, flavor['pool_group'])
 
     def test_get_works(self):
         result = self.simulate_get(self.flavor_path)
@@ -191,9 +192,14 @@ class TestFlavorsMongoDB(base.V2Base):
         flavor = jsonutils.loads(result[0])
         self._flavor_expect(flavor, self.flavor_path, self.doc['pool_group'])
 
-        store_caps = ['FIFO', 'CLAIMS', 'DURABILITY',
-                      'AOD', 'HIGH_THROUGHPUT']
-        self.assertEqual(store_caps, flavor['capabilities'])
+    def test_detailed_get_works(self):
+        result = self.simulate_get(self.flavor_path,
+                                   query_string='detailed=True')
+        self.assertEqual(falcon.HTTP_200, self.srmock.status)
+        flavor = jsonutils.loads(result[0])
+        self._flavor_expect(flavor, self.flavor_path, self.doc['pool_group'])
+        self.assertIn('capabilities', flavor)
+        self.assertEqual({}, flavor['capabilities'])
 
     def test_patch_raises_if_missing_fields(self):
         self.simulate_patch(self.flavor_path,
@@ -201,26 +207,23 @@ class TestFlavorsMongoDB(base.V2Base):
         self.assertEqual(falcon.HTTP_400, self.srmock.status)
 
     def _patch_test(self, doc):
-        result = self.simulate_patch(self.flavor_path,
-                                     body=jsonutils.dumps(doc))
+        self.simulate_patch(self.flavor_path,
+                            body=jsonutils.dumps(doc))
         self.assertEqual(falcon.HTTP_200, self.srmock.status)
-        updated_flavor = jsonutils.loads(result[0])
-        self._flavor_expect(updated_flavor, self.flavor_path,
-                            doc['pool_group'])
-        self.assertEqual(doc['capabilities'], updated_flavor['capabilities'])
 
-        result = self.simulate_get(self.flavor_path)
+        result = self.simulate_get(self.flavor_path,
+                                   query_string='detailed=True')
         self.assertEqual(falcon.HTTP_200, self.srmock.status)
         flavor = jsonutils.loads(result[0])
         self._flavor_expect(flavor, self.flavor_path, doc['pool_group'])
         self.assertEqual(doc['capabilities'], flavor['capabilities'])
 
     def test_patch_works(self):
-        doc = {'pool_group': 'mypoolgroup', 'capabilities': []}
+        doc = {'pool_group': 'my-pool-group', 'capabilities': {'a': 1}}
         self._patch_test(doc)
 
     def test_patch_works_with_extra_fields(self):
-        doc = {'pool_group': 'mypoolgroup', 'capabilities': [],
+        doc = {'pool_group': 'my-pool-group', 'capabilities': {'a': 1},
                'location': 100, 'partition': 'taco'}
         self._patch_test(doc)
 
@@ -239,7 +242,7 @@ class TestFlavorsMongoDB(base.V2Base):
     def test_patch_raises_404_if_flavor_not_found(self):
         self.simulate_patch(self.url_prefix + '/flavors/notexists',
                             body=jsonutils.dumps({'pool_group': 'test'}))
-        self.assertEqual(falcon.HTTP_404, self.srmock.status)
+        self.assertEqual(self.srmock.status, falcon.HTTP_404)
 
     def test_empty_listing(self):
         self.simulate_delete(self.flavor_path)
@@ -293,9 +296,7 @@ class TestFlavorsMongoDB(base.V2Base):
             self.assertEqual(min(limit, count), len(flavors_list))
             for i, s in enumerate(flavors_list + next_flavors_list):
                 expect = expected[i]
-                path = expect[0]
-                capabilities = ['FIFO', 'CLAIMS', 'DURABILITY',
-                                'AOD', 'HIGH_THROUGHPUT']
+                path, capabilities = expect[:2]
                 self._flavor_expect(s, path, self.doc['pool_group'])
                 if detailed:
                     self.assertIn('capabilities', s)
