@@ -14,13 +14,13 @@
 # limitations under the License.
 
 import datetime
-import uuid
 
 import ddt
 import falcon
 import mock
 from oslo_serialization import jsonutils
 from oslo_utils import timeutils
+from oslo_utils import uuidutils
 import six
 from testtools import matchers
 
@@ -44,16 +44,16 @@ class TestMessagesMongoDB(base.V2Base):
             for i in range(4):
                 db_name = "zaqar_test_pools_" + str(i)
                 # NOTE(dynarro): we need to create a unique uri.
-                uri = "%s/%s" % (uri, db_name)
+                new_uri = "%s/%s" % (uri, db_name)
                 options = {'database': db_name}
-                doc = {'weight': 100, 'uri': uri, 'options': options}
+                doc = {'weight': 100, 'uri': new_uri, 'options': options}
                 self.simulate_put(self.url_prefix + '/pools/' + str(i),
                                   body=jsonutils.dumps(doc))
                 self.assertEqual(falcon.HTTP_201, self.srmock.status)
 
         self.project_id = '7e55e1a7e'
         self.headers.update({
-            'Client-ID': str(uuid.uuid4()),
+            'Client-ID': uuidutils.generate_uuid(),
             'X-Project-ID': self.project_id
         })
 
@@ -325,7 +325,7 @@ class TestMessagesMongoDB(base.V2Base):
 
         self.assertEqual(falcon.HTTP_400, self.srmock.status)
 
-    @ddt.data(None, '[', '[]', '{}', '.')
+    @ddt.data(None, '[', '[]', '{}', '.', '123')
     def test_post_bad_message(self, document):
         self.simulate_post(self.queue_path + '/messages',
                            body=document,
@@ -665,7 +665,7 @@ class TestMessagesFaultyDriver(base.V2BaseFaulty):
         path = self.url_prefix + '/queues/fizbit/messages'
         body = '{"messages": [{"body": 239, "ttl": 100}]}'
         headers = {
-            'Client-ID': str(uuid.uuid4()),
+            'Client-ID': uuidutils.generate_uuid(),
             'X-Project-ID': project_id
         }
 
