@@ -16,83 +16,29 @@
 
 import datetime
 import re
+import uuid
 
-from oslo_config import cfg
 from oslo_utils import timeutils
 import six
 
+from zaqar.common import consts
+from zaqar.conf import transport
 from zaqar.i18n import _
 
+
 MIN_MESSAGE_TTL = 60
 MIN_CLAIM_TTL = 60
 MIN_CLAIM_GRACE = 60
+MIN_DELAY_TTL = 0
 MIN_SUBSCRIPTION_TTL = 60
-
-_TRANSPORT_LIMITS_OPTIONS = (
-    cfg.IntOpt('max_queues_per_page', default=20,
-               deprecated_name='queue_paging_uplimit',
-               deprecated_group='limits:transport',
-               help='Defines the maximum number of queues per page.'),
-
-    cfg.IntOpt('max_messages_per_page', default=20,
-               deprecated_name='message_paging_uplimit',
-               deprecated_group='limits:transport',
-               help='Defines the maximum number of messages per page.'),
-
-    cfg.IntOpt('max_subscriptions_per_page', default=20,
-               deprecated_name='subscription_paging_uplimit',
-               deprecated_group='limits:transport',
-               help='Defines the maximum number of subscriptions per page.'),
-
-    cfg.IntOpt('max_messages_per_claim_or_pop', default=20,
-               deprecated_name='max_messages_per_claim',
-               help='The maximum number of messages that can be claimed (OR) '
-                    'popped in a single request'),
-
-    cfg.IntOpt('max_queue_metadata', default=64 * 1024,
-               deprecated_name='metadata_size_uplimit',
-               deprecated_group='limits:transport',
-               help='Defines the maximum amount of metadata in a queue.'),
-
-    cfg.IntOpt('max_messages_post_size', default=256 * 1024,
-               deprecated_name='message_size_uplimit',
-               deprecated_group='limits:transport',
-               deprecated_opts=[cfg.DeprecatedOpt('max_message_size')],
-               help='Defines the maximum size of message posts.'),
-
-    cfg.IntOpt('max_message_ttl', default=1209600,
-               deprecated_name='message_ttl_max',
-               deprecated_group='limits:transport',
-               help='Maximum amount of time a message will be available.'),
-
-    cfg.IntOpt('max_claim_ttl', default=43200,
-               deprecated_name='claim_ttl_max',
-               deprecated_group='limits:transport',
-               help='Maximum length of a message in claimed state.'),
-
-    cfg.IntOpt('max_claim_grace', default=43200,
-               deprecated_name='claim_grace_max',
-               deprecated_group='limits:transport',
-               help='Defines the maximum message grace period in seconds.'),
-
-    cfg.ListOpt('subscriber_types', default=['http', 'https', 'mailto',
-                                             'trust+http', 'trust+https'],
-                help='Defines supported subscriber types.'),
-)
-
-_TRANSPORT_LIMITS_GROUP = 'transport'
-
+_PURGBLE_RESOURCE_TYPES = {'messages', 'subscriptions'}
 # NOTE(kgriffs): Don't use \w because it isn't guaranteed to match
 # only ASCII characters.
-QUEUE_NAME_REGEX = re.compile('^[a-zA-Z0-9_\-]+$')
+QUEUE_NAME_REGEX = re.compile('^[a-zA-Z0-9_\-.]+$')
 QUEUE_NAME_MAX_LEN = 64
 PROJECT_ID_MAX_LEN = 256
 
 
-def _config_options():
-    return [(_TRANSPORT_LIMITS_GROUP, _TRANSPORT_LIMITS_OPTIONS)]
-
-
 class ValidationFailed(ValueError):
     """User input did not follow API restrictions."""
 
@@ -104,9 +50,9 @@ class ValidationFailed(ValueError):
 class Validator(object):
     def __init__(self, conf):
         self._conf = conf
-        self._conf.register_opts(_TRANSPORT_LIMITS_OPTIONS,
-                                 group=_TRANSPORT_LIMITS_GROUP)
-        self._limits_conf = self._conf[_TRANSPORT_LIMITS_GROUP]
+        self._conf.register_opts(transport.ALL_OPTS,
+                                 group=transport.GROUP_NAME)
+        self._limits_conf = self._conf[transport.GROUP_NAME]
         self._supported_operations = ('add', 'remove', 'replace')
 
     def queue_identification(self, queue, project):
@@ -114,7 +60,7 @@ class Validator(object):
 
         :param queue: Name of the queue
         :param project: Project id
-        :raises: ValidationFailed if the `name` is longer than 64
+        :raises ValidationFailed: if the `name` is longer than 64
             characters or contains anything other than ASCII digits and
             letters, underscores, and dashes.  Also raises if `project`
             is not None but longer than 256 characters.
@@ -225,6 +171,49 @@ class Validator(object):
         path_list = self._decode_json_pointer(path)
         return op, path_list
 
+    def _validate_retry_policy(self, metadata):
+        retry_policy = metadata.get('_retry_policy') if metadata else None
+        if retry_policy and not isinstance(retry_policy, dict):
+            msg = _('retry_policy must be a dict.')
+            raise ValidationFailed(msg)
+
+        if retry_policy:
+            valid_keys = ['retries_with_no_delay', 'minimum_delay_retries',
+                          'minimum_delay', 'maximum_delay',
+                          'maximum_delay_retries', 'retry_backoff_function',
+                          'ignore_subscription_override']
+            for key in valid_keys:
+                retry_value = retry_policy.get(key)
+                if key == 'retry_backoff_function':
+                    if retry_value and not isinstance(retry_value, str):
+                        msg = _('retry_backoff_function must be a string.')
+                        raise ValidationFailed(msg)
+                    # Now we support linear, arithmetic, exponential
+                    # and geometric retry backoff function.
+                    fun = {'linear', 'arithmetic', 'exponential', 'geometric'}
+                    if retry_value and retry_value not in fun:
+                        msg = _('invalid retry_backoff_function.')
+                        raise ValidationFailed(msg)
+                elif key == 'ignore_subscription_override':
+                    if retry_value and not isinstance(retry_value, bool):
+                        msg = _('ignore_subscription_override must be a '
+                                'boolean.')
+                        raise ValidationFailed(msg)
+                else:
+                    if retry_value and not isinstance(retry_value, int):
+                        msg = _('Retry policy: %s must be a integer.') % key
+                        raise ValidationFailed(msg)
+            min_delay = retry_policy.get('minimum_delay',
+                                         consts.MINIMUM_DELAY)
+            max_delay = retry_policy.get('maximum_delay',
+                                         consts.MAXIMUM_DELAY)
+            if max_delay < min_delay:
+                msg = _('minimum_delay must less than maximum_delay.')
+                raise ValidationFailed(msg)
+            if ((max_delay - min_delay) < 2*consts.LINEAR_INTERVAL):
+                msg = _('invalid minimum_delay and maximum_delay.')
+                raise ValidationFailed(msg)
+
     def queue_patching(self, request, changes):
         washed_changes = []
         content_types = {
@@ -264,7 +253,7 @@ class Validator(object):
 
         :param limit: The expected number of queues in the list
         :param kwargs: Ignored arguments passed to storage API
-        :raises: ValidationFailed if the limit is exceeded
+        :raises ValidationFailed: if the limit is exceeded
         """
 
         uplimit = self._limits_conf.max_queues_per_page
@@ -276,7 +265,7 @@ class Validator(object):
         """Restrictions on queue's length.
 
         :param content_length: Queue request's length.
-        :raises: ValidationFailed if the metadata is oversize.
+        :raises ValidationFailed: if the metadata is oversize.
         """
         if content_length is None:
             return
@@ -288,17 +277,17 @@ class Validator(object):
         """Checking if the reserved attributes of the queue are valid.
 
         :param queue_metadata: Queue's metadata.
-        :raises: ValidationFailed if any reserved attribute is invalid.
+        :raises ValidationFailed: if any reserved attribute is invalid.
         """
         if not queue_metadata:
             return
 
-        queue_default_ttl = queue_metadata.get('_default_message_ttl', None)
+        queue_default_ttl = queue_metadata.get('_default_message_ttl')
         if queue_default_ttl and not isinstance(queue_default_ttl, int):
             msg = _(u'_default_message_ttl must be integer.')
             raise ValidationFailed(msg)
 
-        if queue_default_ttl:
+        if queue_default_ttl is not None:
             if not (MIN_MESSAGE_TTL <= queue_default_ttl <=
                     self._limits_conf.max_message_ttl):
                 msg = _(u'_default_message_ttl can not exceed {0} '
@@ -312,7 +301,7 @@ class Validator(object):
             msg = _(u'_max_messages_post_size must be integer.')
             raise ValidationFailed(msg)
 
-        if queue_max_msg_size:
+        if queue_max_msg_size is not None:
             if not (0 < queue_max_msg_size <=
                     self._limits_conf.max_messages_post_size):
                 raise ValidationFailed(
@@ -320,11 +309,61 @@ class Validator(object):
                       ' and must be at least greater than 0.'),
                     self._limits_conf.max_messages_post_size)
 
+        max_claim_count = queue_metadata.get('_max_claim_count', None)
+        if max_claim_count and not isinstance(max_claim_count, int):
+            msg = _(u'_max_claim_count must be integer.')
+            raise ValidationFailed(msg)
+
+        dlq_ttl = queue_metadata.get('_dead_letter_queue_messages_ttl', None)
+        if dlq_ttl and not isinstance(dlq_ttl, int):
+            msg = _(u'_dead_letter_queue_messages_ttl must be integer.')
+            raise ValidationFailed(msg)
+
+        if dlq_ttl is not None and not (MIN_MESSAGE_TTL <= dlq_ttl <=
+                                        self._limits_conf.max_message_ttl):
+            msg = _(u'The TTL for a message may not exceed {0} seconds, '
+                    'and must be at least {1} seconds long.')
+            raise ValidationFailed(msg, self._limits_conf.max_message_ttl,
+                                   MIN_MESSAGE_TTL)
+
+        queue_delay = queue_metadata.get('_default_message_delay',
+                                         None)
+        if queue_delay and not isinstance(queue_delay, int):
+            msg = _(u'_default_message_delay must be integer.')
+            raise ValidationFailed(msg)
+
+        if queue_delay is not None:
+            if not (MIN_DELAY_TTL <= queue_delay <=
+                    self._limits_conf.max_message_delay):
+                msg = _(u'The TTL can not exceed {0} seconds, and must '
+                        'be at least {1} seconds long.')
+                raise ValidationFailed(
+                    msg, self._limits_conf.max_message_delay,
+                    MIN_DELAY_TTL)
+
+        self._validate_retry_policy(queue_metadata)
+
+    def queue_purging(self, document):
+        """Restrictions the resource types to be purged for a queue.
+
+        :param resource_types: Type list of all resource under a queue
+        :raises ValidationFailed: if the resource types are invalid
+        """
+
+        if 'resource_types' not in document:
+            msg = _(u'Post body must contain key "resource_types".')
+            raise ValidationFailed(msg)
+
+        if (not set(document['resource_types']).issubset(
+                _PURGBLE_RESOURCE_TYPES)):
+            msg = _(u'Resource types must be a sub set of {0}.')
+            raise ValidationFailed(msg, _PURGBLE_RESOURCE_TYPES)
+
     def message_posting(self, messages):
         """Restrictions on a list of messages.
 
         :param messages: A list of messages
-        :raises: ValidationFailed if any message has a out-of-range
+        :raises ValidationFailed: if any message has a out-of-range
             TTL.
         """
 
@@ -338,7 +377,7 @@ class Validator(object):
         """Restrictions on message post length.
 
         :param content_length: Queue request's length.
-        :raises: ValidationFailed if the metadata is oversize.
+        :raises ValidationFailed: if the metadata is oversize.
         """
         if content_length is None:
             return
@@ -379,12 +418,23 @@ class Validator(object):
             raise ValidationFailed(
                 msg, self._limits_conf.max_message_ttl, MIN_MESSAGE_TTL)
 
+        delay = message.get('delay', 0)
+
+        if not (MIN_DELAY_TTL <= delay <=
+                self._limits_conf.max_message_delay):
+            msg = _(u'The Delay TTL for a message may not exceed {0} seconds,'
+                    'and must be at least {1} seconds long.')
+
+            raise ValidationFailed(
+                msg, self._limits_conf.max_message_delay,
+                MIN_DELAY_TTL)
+
     def message_listing(self, limit=None, **kwargs):
         """Restrictions involving a list of messages.
 
         :param limit: The expected number of messages in the list
         :param kwargs: Ignored arguments passed to storage API
-        :raises: ValidationFailed if the limit is exceeded
+        :raises ValidationFailed: if the limit is exceeded
         """
 
         uplimit = self._limits_conf.max_messages_per_page
@@ -400,10 +450,10 @@ class Validator(object):
 
         :param ids: message ids passed in by the delete request
         :param pop: count of messages to be POPped
-        :raises: ValidationFailed if,
-                 pop AND id params are present together
-                 neither pop or id params are present
-                 message count to be popped > maximum allowed
+        :raises ValidationFailed: if,
+            pop AND id params are present together
+            neither pop or id params are present
+            message count to be popped > maximum allowed
         """
 
         if pop is not None and ids is not None:
@@ -437,7 +487,7 @@ class Validator(object):
 
         :param metadata: The claim metadata
         :param limit: The number of messages to claim
-        :raises: ValidationFailed if either TTL or grace is out of range,
+        :raises ValidationFailed: if either TTL or grace is out of range,
             or the expected number of messages exceed the limit.
         """
 
@@ -464,7 +514,7 @@ class Validator(object):
         """Restrictions on the claim TTL.
 
         :param metadata: The claim metadata
-        :raises: ValidationFailed if the TTL is out of range
+        :raises ValidationFailed: if the TTL is out of range
         """
 
         ttl = metadata['ttl']
@@ -480,7 +530,7 @@ class Validator(object):
         """Restrictions on a creation of subscription.
 
         :param subscription: dict of subscription
-        :raises: ValidationFailed if the subscription is invalid.
+        :raises ValidationFailed: if the subscription is invalid.
         """
         for p in ('subscriber',):
             if p not in subscription.keys():
@@ -492,13 +542,17 @@ class Validator(object):
         """Restrictions on an update of subscription.
 
         :param subscription: dict of subscription
-        :raises: ValidationFailed if the subscription is invalid.
+        :raises ValidationFailed: if the subscription is invalid.
         """
 
         if not subscription:
             raise ValidationFailed(_(u'No subscription to create.'))
 
-        subscriber = subscription.get('subscriber', None)
+        if not isinstance(subscription, dict):
+            msg = _('Subscriptions must be a dict.')
+            raise ValidationFailed(msg)
+
+        subscriber = subscription.get('subscriber')
         subscriber_type = None
 
         if subscriber:
@@ -510,12 +564,14 @@ class Validator(object):
                         u'supported in the list {0}.')
                 raise ValidationFailed(msg, self._limits_conf.subscriber_types)
 
-        options = subscription.get('options', None)
+        options = subscription.get('options')
         if options and not isinstance(options, dict):
             msg = _(u'Options must be a dict.')
             raise ValidationFailed(msg)
 
-        ttl = subscription.get('ttl', None)
+        self._validate_retry_policy(options)
+
+        ttl = subscription.get('ttl')
         if ttl:
             if not isinstance(ttl, int):
                 msg = _(u'TTL must be an integer.')
@@ -540,9 +596,9 @@ class Validator(object):
             except OverflowError:
                 raise ValidationFailed(msg, datetime.datetime.max)
 
-    def subscription_confirming(self, confirm):
-        confirm = confirm.get('confirmed', None)
-        if not isinstance(confirm, bool):
+    def subscription_confirming(self, confirmed):
+        confirmed = confirmed.get('confirmed')
+        if not isinstance(confirmed, bool):
             msg = _(u"The 'confirmed' should be boolean.")
             raise ValidationFailed(msg)
 
@@ -551,7 +607,7 @@ class Validator(object):
 
         :param limit: The expected number of subscriptions in the list
         :param kwargs: Ignored arguments passed to storage API
-        :raises: ValidationFailed if the limit is exceeded
+        :raises ValidationFailed: if the limit is exceeded
         """
 
         uplimit = self._limits_conf.max_subscriptions_per_page
@@ -568,3 +624,47 @@ class Validator(object):
         :param limit_conf_name: configuration name
         """
         return self._limits_conf[limit_conf_name]
+
+    def flavor_listing(self, limit=None, **kwargs):
+        """Restrictions involving a list of pools.
+
+        :param limit: The expected number of flavors in the list
+        :param kwargs: Ignored arguments passed to storage API
+        :raises ValidationFailed: if the limit is exceeded
+        """
+
+        uplimit = self._limits_conf.max_flavors_per_page
+        if limit is not None and not (0 < limit <= uplimit):
+            msg = _(u'Limit must be at least 1 and no greater than {0}.')
+            raise ValidationFailed(msg, self._limits_conf.max_flavors_per_page)
+
+    def pool_listing(self, limit=None, **kwargs):
+        """Restrictions involving a list of pools.
+
+        :param limit: The expected number of flavors in the list
+        :param kwargs: Ignored arguments passed to storage API
+        :raises ValidationFailed: if the limit is exceeded
+        """
+
+        uplimit = self._limits_conf.max_pools_per_page
+        if limit is not None and not (0 < limit <= uplimit):
+            msg = _(u'Limit must be at least 1 and no greater than {0}.')
+            raise ValidationFailed(msg, self._limits_conf.max_pools_per_page)
+
+    def client_id_uuid_safe(self, client_id):
+        """Restrictions the format of client id
+
+        :param client_id: the client id of request
+        :raises ValidationFailed: if the limit is exceeded
+        """
+
+        if self._limits_conf.client_id_uuid_safe == 'off':
+            if (len(client_id) < self._limits_conf.min_length_client_id) or \
+               (len(client_id) > self._limits_conf.max_length_client_id):
+                msg = _(u'Length of client id must be at least {0} and no '
+                        'greater than {1}.')
+                raise ValidationFailed(msg,
+                                       self._limits_conf.min_length_client_id,
+                                       self._limits_conf.max_length_client_id)
+        if self._limits_conf.client_id_uuid_safe == 'strict':
+            uuid.UUID(client_id)
