@@ -14,44 +14,33 @@
 # limitations under the License.
 
 from distutils import version as d_version
-from wsgiref import simple_server
-
 import falcon
-from oslo_config import cfg
-from oslo_log import log as logging
-from oslo_utils import netutils
 import six
 import socket
+from wsgiref import simple_server
+
+from oslo_log import log as logging
+from oslo_utils import netutils
 
 from zaqar.common import decorators
 from zaqar.common.transport.wsgi import helpers
+from zaqar.conf import drivers_transport_wsgi
 from zaqar.i18n import _
 from zaqar import transport
 from zaqar.transport import acl
-from zaqar.transport import auth
+from zaqar.transport.middleware import auth
+from zaqar.transport.middleware import cors
+from zaqar.transport.middleware import profile
 from zaqar.transport import validation
 from zaqar.transport.wsgi import v1_0
 from zaqar.transport.wsgi import v1_1
 from zaqar.transport.wsgi import v2_0
 from zaqar.transport.wsgi import version
 
-_WSGI_OPTIONS = (
-    cfg.IPOpt('bind', default='127.0.0.1',
-              help='Address on which the self-hosting server will listen.'),
-
-    cfg.PortOpt('port', default=8888,
-                help='Port on which the self-hosting server will listen.'),
-)
-
-_WSGI_GROUP = 'drivers:transport:wsgi'
 
 LOG = logging.getLogger(__name__)
 
 
-def _config_options():
-    return [(_WSGI_GROUP, _WSGI_OPTIONS)]
-
-
 class FuncMiddleware(object):
 
     def __init__(self, func):
@@ -66,8 +55,9 @@ class Driver(transport.DriverBase):
     def __init__(self, conf, storage, cache, control):
         super(Driver, self).__init__(conf, storage, cache, control)
 
-        self._conf.register_opts(_WSGI_OPTIONS, group=_WSGI_GROUP)
-        self._wsgi_conf = self._conf[_WSGI_GROUP]
+        self._conf.register_opts(drivers_transport_wsgi.ALL_OPTS,
+                                 group=drivers_transport_wsgi.GROUP_NAME)
+        self._wsgi_conf = self._conf[drivers_transport_wsgi.GROUP_NAME]
         self._validate = validation.Validator(self._conf)
 
         self.app = None
@@ -82,6 +72,10 @@ class Driver(transport.DriverBase):
         return helpers.validate_queue_identification(
             self._validate.queue_identification, req, resp, params)
 
+    def _require_client_id(self, req, resp, params):
+        return helpers.require_client_id(
+            self._validate.client_id_uuid_safe, req, resp, params)
+
     @decorators.lazy_property(write=False)
     def before_hooks(self):
         """Exposed to facilitate unit testing."""
@@ -89,7 +83,7 @@ class Driver(transport.DriverBase):
             self._verify_pre_signed_url,
             helpers.require_content_type_be_non_urlencoded,
             helpers.require_accepts_json,
-            helpers.require_client_id,
+            self._require_client_id,
             helpers.extract_project_id,
 
             # NOTE(jeffrey4l): Depends on the project_id and client_id being
@@ -138,8 +132,12 @@ class Driver(transport.DriverBase):
     def _init_middleware(self):
         """Initialize WSGI middlewarez."""
 
-        auth_app = self.app
+        # NOTE(zhiyan): Install Profiler
+        if (self._conf.profiler.enabled and
+                self._conf.profiler.trace_wsgi_transport):
+            self.app = profile.install_wsgi_tracer(self.app, self._conf)
 
+        auth_app = self.app
         # NOTE(flaper87): Install Auth
         if self._conf.auth_strategy:
             strategy = auth.strategy(self._conf.auth_strategy)
@@ -147,6 +145,10 @@ class Driver(transport.DriverBase):
 
         self.app = auth.SignedHeadersAuth(self.app, auth_app)
 
+        # NOTE(wangxiyuan): Install CORS, this middleware should be called
+        # before Keystone auth.
+        self.app = cors.install_cors(self.app, auth_app, self._conf)
+
         acl.setup_policy(self._conf)
 
     def _error_handler(self, exc, request, response, params):
