@@ -20,6 +20,7 @@ from zaqar.transport.wsgi.v2_0 import homedoc
 from zaqar.transport.wsgi.v2_0 import messages
 from zaqar.transport.wsgi.v2_0 import ping
 from zaqar.transport.wsgi.v2_0 import pools
+from zaqar.transport.wsgi.v2_0 import purge
 from zaqar.transport.wsgi.v2_0 import queues
 from zaqar.transport.wsgi.v2_0 import stats
 from zaqar.transport.wsgi.v2_0 import subscriptions
@@ -69,7 +70,8 @@ def public_endpoints(driver, conf):
                              message_controller)),
         ('/queues/{queue_name}/stats',
          stats.Resource(queue_controller)),
-
+        ('/queues/{queue_name}/purge',
+         purge.Resource(driver)),
         # Messages Endpoints
         ('/queues/{queue_name}/messages',
          messages.CollectionResource(driver._wsgi_conf,
@@ -112,7 +114,8 @@ def public_endpoints(driver, conf):
 
         ('/queues/{queue_name}/subscriptions/{subscription_id}/confirm',
          subscriptions.ConfirmResource(driver._validate,
-                                       subscription_controller)),
+                                       subscription_controller,
+                                       conf)),
 
         # Pre-Signed URL Endpoint
         ('/queues/{queue_name}/share', urls.Resource(driver)),
@@ -131,14 +134,16 @@ def private_endpoints(driver, conf):
     if conf.pooling:
         pools_controller = driver._control.pools_controller
         flavors_controller = driver._control.flavors_controller
+        validate = driver._validate
 
         catalogue.extend([
             ('/pools',
-             pools.Listing(pools_controller)),
+             pools.Listing(pools_controller, validate)),
             ('/pools/{pool}',
              pools.Resource(pools_controller)),
             ('/flavors',
-             flavors.Listing(flavors_controller, pools_controller)),
+             flavors.Listing(flavors_controller, pools_controller,
+                             validate)),
             ('/flavors/{flavor}',
              flavors.Resource(flavors_controller, pools_controller)),
         ])
