@@ -43,7 +43,7 @@ class ItemResource(object):
         self._validate = validate
         self._subscription_controller = subscription_controller
 
-    @decorators.TransportLog("Subscription")
+    @decorators.TransportLog("Subscriptions item")
     @acl.enforce("subscription:get")
     def on_get(self, req, resp, project_id, queue_name, subscription_id):
         try:
@@ -63,7 +63,7 @@ class ItemResource(object):
         resp.body = utils.to_json(resp_dict)
         # status defaults to 200
 
-    @decorators.TransportLog("Subscription")
+    @decorators.TransportLog("Subscriptions item")
     @acl.enforce("subscription:delete")
     def on_delete(self, req, resp, project_id, queue_name, subscription_id):
         try:
@@ -78,7 +78,7 @@ class ItemResource(object):
 
         resp.status = falcon.HTTP_204
 
-    @decorators.TransportLog("Subscription")
+    @decorators.TransportLog("Subscriptions item")
     @acl.enforce("subscription:update")
     def on_patch(self, req, resp, project_id, queue_name, subscription_id):
         if req.content_length:
@@ -126,7 +126,7 @@ class CollectionResource(object):
         self._conf = conf
         self._notification = notifier.NotifierDriver()
 
-    @decorators.TransportLog("Subscription collection")
+    @decorators.TransportLog("Subscriptions collection")
     @acl.enforce("subscription:get_all")
     def on_get(self, req, resp, project_id, queue_name):
         kwargs = {}
@@ -171,7 +171,7 @@ class CollectionResource(object):
         resp.body = utils.to_json(response_body)
         # status defaults to 200
 
-    @decorators.TransportLog("Subscription item")
+    @decorators.TransportLog("Subscriptions collection")
     @acl.enforce("subscription:create")
     def on_post(self, req, resp, project_id, queue_name):
         if req.content_length:
@@ -248,13 +248,16 @@ class CollectionResource(object):
 
 class ConfirmResource(object):
 
-    __slots__ = ('_subscription_controller', '_validate')
+    __slots__ = ('_subscription_controller', '_validate', '_notification',
+                 '_conf')
 
-    def __init__(self, validate, subscription_controller):
+    def __init__(self, validate, subscription_controller, conf):
         self._subscription_controller = subscription_controller
         self._validate = validate
+        self._notification = notifier.NotifierDriver()
+        self._conf = conf
 
-    @decorators.TransportLog("Confirm Subscription")
+    @decorators.TransportLog("Subscriptions confirmation item")
     @acl.enforce("subscription:confirm")
     def on_put(self, req, resp, project_id, queue_name, subscription_id):
         if req.content_length:
@@ -264,10 +267,26 @@ class ConfirmResource(object):
 
         try:
             self._validate.subscription_confirming(document)
-            confirm = document.get('confirmed', None)
+            confirmed = document.get('confirmed')
             self._subscription_controller.confirm(queue_name, subscription_id,
                                                   project=project_id,
-                                                  confirm=confirm)
+                                                  confirmed=confirmed)
+            if confirmed is False:
+                now = timeutils.utcnow_ts()
+                now_dt = datetime.datetime.utcfromtimestamp(now)
+                ttl = self._conf.transport.default_subscription_ttl
+                expires = now_dt + datetime.timedelta(seconds=ttl)
+                api_version = req.path.split('/')[1]
+                sub = self._subscription_controller.get(queue_name,
+                                                        subscription_id,
+                                                        project=project_id)
+                self._notification.send_confirm_notification(queue_name,
+                                                             sub,
+                                                             self._conf,
+                                                             project_id,
+                                                             str(expires),
+                                                             api_version,
+                                                             True)
             resp.status = falcon.HTTP_204
             resp.location = req.path
         except storage_errors.SubscriptionDoesNotExist as ex:
